/* dc_gui2 - a GTK+2 GUI for DCTC
 * Copyright (C) 2002 Eric Prevoteau
 *
 * user_clist.c: Copyright (C) Eric Prevoteau <www@a2pb.gotdns.org>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */
/*
$Id: user_clist.c,v 1.5 2003/12/02 18:01:05 ericprev Exp $
*/

#ifdef HAVE_CONFIG_H
#  include <config.h>
#endif

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <gnome.h>

#include "gtkcellrenderertextx.h"
#include "misc_gtk.h"
#include "user_clist.h"
#include "main.h"
#include "gui_define.h"
#include "macro.h"
#include "global_user.h"

/************************************************/
/* update user information using GLOB_USER data */
/************************************************/
static void user_clist_user_info_update_row(GtkTreeModel *gtm, GtkTreePath *gtp, GtkTreeIter *gti, gpointer user_data)
{
	GLOB_USER *gu;

	/* get GLOB_USER entry to use */
	gtk_tree_model_get(gtm,gti,UCC_USER_POINTER,&gu,-1);

	/* block the update signal on this function */
	g_signal_handlers_block_matched (G_OBJECT(gtm),G_SIGNAL_MATCH_FUNC,0,0,NULL,user_clist_user_info_update_row,NULL);

	/* update user info */
	gtk_list_store_set(GTK_LIST_STORE(gtm),gti,
												UCC_TYPE_COL,gu->cnx_type,
												UCC_NICK_COL,gu->fmt_nick,
												UCC_SIZE_COL,gu->fmt_size,
												UCC_EMAIL_COL,gu->fmt_email,
												UCC_DESCRIPTION_COL,gu->fmt_desc,
												UCC_LINE_BACKGROUND,gu->back_color,
												UCC_LINE_FOREGROUND,gu->front_color,
												UCC_TYPE_CNX_BACKGROUND,gu->fast_color,
												-1);

	/* unblock the update signal on this function */
	g_signal_handlers_unblock_matched (G_OBJECT(gtm),G_SIGNAL_MATCH_FUNC,0,0,NULL,user_clist_user_info_update_row,NULL);
}

/********************************************/
/* build modele and view for the user clist */
/********************************************/
void bmav4_user_clist(void)
{
	GtkListStore *model;
	GtkWidget *view;
	GtkCellRenderer *rend;
	GtkTreeSelection *slc;

	model=gtk_list_store_new(NB_UCC_COL,G_TYPE_STRING,G_TYPE_STRING,G_TYPE_STRING,
													G_TYPE_STRING,G_TYPE_STRING,G_TYPE_FLOAT,
													G_TYPE_STRING,G_TYPE_STRING,G_TYPE_STRING,
													G_TYPE_POINTER, G_TYPE_UINT64);
	g_signal_connect_after(G_OBJECT(model),"row-changed",G_CALLBACK(user_clist_user_info_update_row),NULL);
	view=get_widget_by_widget_name(main_window,"user_clist");

	gtk_tree_view_set_model(GTK_TREE_VIEW(view),GTK_TREE_MODEL(model));
	gtk_tree_view_set_headers_visible(GTK_TREE_VIEW(view),TRUE);

	slc=gtk_tree_view_get_selection(GTK_TREE_VIEW(view));
	gtk_tree_selection_set_mode(slc,GTK_SELECTION_SINGLE);

	/* the first column of the view display the first column of the model, and so on. The 4th entry of the model is not displayed */
	rend=gtk_cell_renderer_textx_new();
	g_object_set(rend,"rowspacing",TRUE,NULL);
	MY_RS_gtk_tree_view_insert_column_with_attributes(GTK_TREE_VIEW(view),UCC_TYPE_COL,_("Spd"),rend,"text",UCC_TYPE_COL,"background",UCC_TYPE_CNX_BACKGROUND,"foreground",UCC_LINE_FOREGROUND,NULL);

	rend=gtk_cell_renderer_textx_new();
	g_object_set(rend,"rowspacing",TRUE,NULL);
	MY_RS_gtk_tree_view_insert_column_with_attributes(GTK_TREE_VIEW(view),UCC_NICK_COL,_("Nick"),rend,"text",UCC_NICK_COL,"background",UCC_LINE_BACKGROUND,"foreground",UCC_LINE_FOREGROUND,NULL);

	rend=gtk_cell_renderer_textx_new();
	g_object_set(rend,"rowspacing",TRUE,NULL);
	MY_RS2_gtk_tree_view_insert_column_with_attributes(GTK_TREE_VIEW(view),UCC_SIZE_COL,UCC_SIZE_AS_VAL,_("Size"),rend,"text",UCC_SIZE_COL,"xalign",UCC_SIZE_RIGHT_ALIGN,"background",UCC_LINE_BACKGROUND,"foreground",UCC_LINE_FOREGROUND,NULL);

	rend=gtk_cell_renderer_textx_new();
	g_object_set(rend,"rowspacing",TRUE,NULL);
	MY_RS_gtk_tree_view_insert_column_with_attributes(GTK_TREE_VIEW(view),UCC_EMAIL_COL,_("e-mail"),rend,"text",UCC_EMAIL_COL,"background",UCC_LINE_BACKGROUND,"foreground",UCC_LINE_FOREGROUND,NULL);

	rend=gtk_cell_renderer_textx_new();
	g_object_set(rend,"rowspacing",TRUE,NULL);
	MY_RS_gtk_tree_view_insert_column_with_attributes(GTK_TREE_VIEW(view),UCC_DESCRIPTION_COL,_("description"),rend,"text",UCC_DESCRIPTION_COL,"background",UCC_LINE_BACKGROUND,"foreground",UCC_LINE_FOREGROUND,NULL);

	/* The view now holds a reference.  We can get rid of our own
	 * reference */
	g_object_unref (G_OBJECT (model));
}

/* ------------------------------------------------------------------------------ */
typedef struct
{
	int fnd;
	GString *fnd_nick;
	int nick_len;
	const char *nick_to_found;
} TMP_NICKCOMP;

/**************************************************/
/* compare a given row with the searched nickname */
/**************************************************/
static gboolean search_completion(GtkTreeModel *model, GtkTreePath *path, GtkTreeIter *iter, gpointer data)
{
	char *nickname;
	TMP_NICKCOMP *tn=data;
	gboolean ret=FALSE;

	gtk_tree_model_get(model,iter,UCC_NICK_COL,&nickname,-1);	/* it is an exception because we complete a nickname in UTF8, not an ASCII one */
	if(nickname)
	{
		if(!strncmp(nickname,tn->nick_to_found,tn->nick_len))
		{
			/* the pattern matchs */
			if(tn->fnd==-1)
			{
				/* not found yet */
				g_string_assign(tn->fnd_nick,nickname);
				tn->fnd=0;
				/* keep searching */
			}
			else if(tn->fnd==0)
			{
				/* already found */
				g_string_assign(tn->fnd_nick,"");
				tn->fnd=-2;
				ret=TRUE;		/* abort search */
			}
		}
		free(nickname);
	}
	return ret;
}

/*********************************************************************************************/
/* this function search in the user_clist to find a nick matching the partially provided one */
/*********************************************************************************************/
/* output: a Gstring containing the full nick or NULL if none or more than one match */
/*************************************************************************************/
GString *complete_a_nick(const char *partial_nick)
{
	TMP_NICKCOMP tn;
	GtkWidget *w;
	GtkTreeModel *gtm;

	w=get_widget_by_widget_name(main_window,"user_clist");
	if(w==NULL)
		return NULL;

	gtm=gtk_tree_view_get_model(GTK_TREE_VIEW(w));

	tn.fnd=-1;	  /* not found */
	tn.fnd_nick=g_string_new("");
	tn.nick_to_found=partial_nick;
	tn.nick_len=strlen(tn.nick_to_found);

	gtk_tree_model_foreach(gtm,search_completion,&tn);

	if(tn.fnd_nick->len>0)
	{
		return tn.fnd_nick;
	}
	g_string_free(tn.fnd_nick,TRUE);
	return NULL;
}
/* ------------------------------------------------------------------------------ */


